@@ -1,12 +1,11 @@
-'''
-Simple script that '''
 import re
 import shlex
 import sys
-# import traceback
 import subprocess
 import requests
+import argparse
 from InquirerPy import inquirer
+from typing import Union, Tuple
 
 base_url: str = 'https://www1.gogoanime.cm'
 program: str = 'mpv'
@@ -17,7 +16,7 @@ header: dict = {
 
 session: requests.Session = requests.session()
 
-def search(name: str) -> list:
+def search(name: str) -> dict:
     '''
     Scrape anime from gogoanime search page
     params:
@@ -36,7 +35,7 @@ def search(name: str) -> list:
         d[match[1]] = match[0]
     return d
 
-def get_episode_count(anime_id: str) -> int:
+def get_episode_count(anime_id: str) -> Union[int, None]:
     '''
     Scrape episode avaliable count
     params:
@@ -48,11 +47,11 @@ def get_episode_count(anime_id: str) -> int:
     pattern: str = r'''ep_start\s?=\s?['"]([0-9]+)['"]\sep_end\s?=\s?['"]([0-9]+)['"]>'''
     episodes: list = re.findall(pattern, response.text)
     if episodes == []:
-        return 'Not found'
-    episode_count = int(episodes[-1][1])
+        return None
+    episode_count: int = int(episodes[-1][1])
     return episode_count
 
-def get_embed_link(anime_id: str, episode: int) -> str:
+def get_embed_link(anime_id: str, episode: int) -> Union[str, None]:
     '''
     Scrape embed link from gogoanime
     params:
@@ -86,7 +85,22 @@ def get_link(embedded_link: str) -> str:
     link: str = re.search(r"https:.*(m3u8)|(mp4)", link).group()
     return link
 
-def play_episode(anime_id: str, episode: int):
+def get_quality(embed_link: str, link: str) -> list:
+    '''
+    Get available quality
+    params:
+        embed_link (str): gogoanime embedded link
+        link (str): m3u8 link
+    return:
+        qualitys (list): list of quality avaliable
+    '''
+
+    video_file: requests.Response = session.get(link, headers=dict(header, **{'referer': embed_link}))
+    qualitys: list = re.findall(r'([0-9]+)\.m3u8', video_file.text)
+
+    return qualitys
+
+def play_episode(anime_id: str, episode: int, quality: str):
     '''
     Play episode
     params:
@@ -95,11 +109,22 @@ def play_episode(anime_id: str, episode: int):
     return:
         subprocess.popen process
     '''
-    embed_link: str = get_embed_link(anime_id, episode)
+    embed_link = get_embed_link(anime_id, episode)
     if embed_link is None:
-        return print('Error: embed link not found')
+        print('Error: embed link not found')
+        return None
 
     link: str = get_link(embed_link)
+    qualitys = get_quality(embed_link, link)
+    if quality in qualitys:
+        link = link.replace('.m3u8', f'.{quality}.m3u8')
+    elif quality == 'best':
+        link = link.replace('.m3u8', f'.{qualitys[-1]}.m3u8')
+    elif quality == 'worst':
+        link = link.replace('.m3u8', f'.{qualitys[0]}.m3u8')
+    else:
+        print('Specified quality not available, defaulting to best')
+        link = link.replace('.m3u8', f'.{qualitys[-1]}.m3u8')
     process = subprocess.Popen(
         shlex.split(f'mpv --http-header-fields="Referer: {embed_link}" {link}'),
         stdout=subprocess.DEVNULL,
@@ -107,14 +132,14 @@ def play_episode(anime_id: str, episode: int):
     )
     return process
 
-def get_anime(name=None) -> str:
+def get_anime(name=None) -> Union[Tuple[str, str], Tuple[None, None]]:
     '''
     Get anime title from user
     '''
     if name is None:
-        name: str  = inquirer.text(message='Input anime title:').execute()
+        name = inquirer.text(message='Input anime title:').execute()
 
-    search_result: list = search(name)
+    search_result: dict = search(name)
 
     if not search_result:
         print('No result found')
@@ -122,7 +147,7 @@ def get_anime(name=None) -> str:
 
     anime_title: str = inquirer.select(
         message='Select anime to watch',
-        choices=search_result
+        choices=list(search_result.keys())
     ).execute()
 
     anime_id = search_result[anime_title]
@@ -146,20 +171,32 @@ def main():
     '''
     Main
     '''
+    
     try:
-        if len(sys.argv) > 1:
-            anime_title, anime_id = get_anime(' '.join(sys.argv[1:]))
+        parser = argparse.ArgumentParser(description='argparse')
+        parser.add_argument('-q', '--quality', help='Quality to watch', default='best')
+        parser.add_argument('rest', nargs=argparse.REMAINDER)
+
+        args = parser.parse_args()
+
+        if args.rest != []:
+            anime_title, anime_id = get_anime(' '.join(args.rest))
         else:
             anime_title, anime_id = get_anime()
-
-        while anime_title is None:
+        #anime_title, anime_id = get_anime()
+        
+        while anime_title is None or anime_id is None:
             anime_title, anime_id = get_anime()
 
         episode_count = get_episode_count(anime_id)
 
+        if episode_count is None:
+            print('Error getting episode count')
+            sys.exit()
+
         episode = get_episode(episode_count)
 
-        play_episode(anime_id, episode)
+        play_episode(anime_id, episode, args.quality)
 
         action = ''
         while action != 'Quit':
@@ -191,7 +228,6 @@ def main():
     except KeyboardInterrupt:
         print('Exiting...')
         sys.exit()
-    # except Exception:
 
 if __name__ == '__main__':
     main()
